@@ -2,7 +2,7 @@
 * common.h: common definitions
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: common.h,v 1.5 2002/07/12 18:09:36 massiot Exp $
+* $Id: common.h,v 1.5.4.1 2003/02/03 12:26:39 nitrox Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *
@@ -99,8 +99,41 @@ typedef void*               handle;
 // Counter for statistics and profiling
 typedef unsigned long       count;
 
+#ifdef HAVE_STDINT_H
+#   include <stdint.h>
+    typedef uint8_t             u8;
+    typedef int8_t              s8;
 
+    typedef uint16_t            u16;
+    typedef int16_t             s16;
 
+    typedef uint32_t            u32;
+    typedef int32_t             s32;
+
+    typedef uint64_t            u64;
+    typedef int64_t             s64;
+#else
+    typedef unsigned char       u8;
+    typedef signed char         s8;
+
+    typedef unsigned short      u16;
+    typedef signed short        s16;
+
+    typedef unsigned int        u32;
+    typedef signed int          s32;
+
+#   if defined( _MSC_VER ) || ( defined( WIN32 ) && !defined( __MINGW32__ ) )
+    typedef unsigned __int64    u64;
+    typedef signed __int64      s64;
+#   else
+    typedef unsigned long long  u64;
+    typedef signed long long    s64;
+#   endif
+#endif
+
+#if defined( WIN32)
+typedef int                    ssize_t;
+#endif
 //------------------------------------------------------------------------------
 // Byte manipulation macros
 //------------------------------------------------------------------------------
