@@ -2,7 +2,7 @@
 * thread.cpp: thread management
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: thread.cpp,v 1.2 2001/10/07 12:26:45 bozo Exp $
+* $Id: thread.cpp,v 1.7 2002/07/12 18:09:36 massiot Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *
@@ -36,11 +36,14 @@
 #include <errno.h>
 #elif defined WIN32
 #include <windows.h>
+#include <process.h>
 #endif
 
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "common.h"
 #include "string.h"
@@ -266,16 +269,21 @@ void C_Thread::Create()
     pthread_attr_setdetachstate(&sAttr, PTHREAD_CREATE_JOINABLE);
 
     int iRc = pthread_create(&tId, NULL, C_Thread::StartRoutine, this);
+    
     pthread_attr_destroy(&sAttr);
+    
+#elif defined _WIN32
+    unsigned long threadID;
+    hThread = (HANDLE)CreateThread(NULL, 0, 
+            (LPTHREAD_START_ROUTINE)C_Thread::StartRoutine, this, 0, &threadID);
+    int iRc = ( hThread ? 0 : 1 );
+#endif
 
     if(iRc)
     {
       throw E_Exception(GEN_ERR, "Unable to start thread: " + GetErrorMsg());
     }
   
-#elif defined WIN32
-    ASSERT(false);
-#endif
 
     // Reference the thread in the list of running threads
     s_cThreadList.PushEnd(this);
@@ -298,12 +306,9 @@ void C_Thread::Cancel()
     // Ask for thread cancellation
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
     int iRc = pthread_cancel(tId);
-
 #elif defined WIN32
-    ASSERT(false);
-    int iRc = GEN_ERR;
+    int iRc = 0;
 #endif
-
     if(iRc)
       throw E_Exception(iRc, "Thread cancellation failed: " + GetErrorMsg()); 
 
@@ -328,11 +333,16 @@ void C_Thread::Stop()
     // As all our threads are created joinable, we must ensure that the calling
     // thread is a different thread to avoid a deadlock. So if a thread try to stop
     // itself, we ask another thread (the killer) to do the job.
+#ifdef PTHREAD_COND_T_IN_PTHREAD_H
     if(pthread_self() == tId)
+#else
+    if (GetCurrentThread() == hThread)
+#endif
     {
       ASSERT(false);
       //s_cThreadKiller.Kill(this);
     }
+
     // Don't join if not started
     // It avoids exception's generation because pthread_join returns an error
     // when the thread was not started.
@@ -341,7 +351,6 @@ void C_Thread::Stop()
       // Ask the user work routine to stop its normal processing
       StopWork();
   
-#ifdef PTHREAD_COND_T_IN_PTHREAD_H
       // Try to join the thread in order to clean the resources it used and
       // to synchronise the calling thread with the destruction of this thread
       // Seems that there is a bug here in the libpthread: when the thread
@@ -349,7 +358,12 @@ void C_Thread::Stop()
       // the return value of the DoWork method
       // Ok, let's try again to see if the pb still occurs
       E_Exception* p_eError = NULL;
+#ifdef PTHREAD_COND_T_IN_PTHREAD_H
       int iRc = pthread_join(tId, (void**)&p_eError);
+#elif defined _WIN32
+      int iRc = 0;
+      WaitForSingleObject (hThread, INFINITE);
+#endif
       if(iRc)
       {
         // Look at thre return code of the joined thread
@@ -359,10 +373,6 @@ void C_Thread::Stop()
         throw e;
       }
 
-#elif defined WIN32
-    ASSERT(false);
-#endif
-
       // Cleaning
       CleanWork();
 
@@ -388,11 +398,11 @@ void C_Thread::Interrupt()
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
   int iRc = pthread_cancel(tId);
+#elif defined _WIN32
+  int iRc = 0;
+#endif
   if(iRc)
     throw E_Exception(GEN_ERR, "Unable to interrupt thread: " + GetErrorMsg());
-#elif defined WIN32
-  ASSERT(false);
-#endif
   m_iStatus = THREAD_STATUS_INTERRUPTED;
 }
 
@@ -407,7 +417,12 @@ void C_Thread::DelayedInterruption()
     for(unsigned int ui = 0; (ui < INT_TEST_COUNT) &&
                              (m_iStatus == THREAD_STATUS_STARTED); ui++)
     {
+#ifdef WIN32
+      Sleep(INT_TEST_DELAY / 1000000);
+#else
       usleep(INT_TEST_DELAY);
+#endif
+
     }
   }
 
@@ -428,10 +443,8 @@ bool C_Thread::operator == (const C_Thread& cThread)
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
   return (this->tId == cThread.tId);
-
 #elif defined WIN32
-  ASSERT(false);
-  return false;
+  return (this->hThread == cThread.hThread);
 #endif
 };
 
@@ -458,9 +471,17 @@ C_Thread* C_Thread::Self()
       break;
     }
   }
-
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  HANDLE hThread = GetCurrentThread();
+  for(unsigned int i = 0; i < s_cThreadList.Size(); i++)
+  {
+    C_Thread& cCurrent = s_cThreadList[i];
+    if(cCurrent.hThread == hThread)
+    {
+      pResult = &cCurrent;
+      break;
+    }
+  }
 #endif
 
   return pResult;
@@ -523,8 +544,8 @@ C_Mutex::C_Mutex()
   ASSERT(!iRc);         // The function shouldn't fail
   USE(iRc);
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  InitializeCriticalSection( &m_sMutex );
 #endif  
 }
 
@@ -539,8 +560,8 @@ C_Mutex::~C_Mutex()
   ASSERT(!iRc);         // The mutex should not be locked when being destroyed
   USE(iRc);
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  DeleteCriticalSection( &m_sMutex );
 #endif
 }
 
@@ -555,8 +576,8 @@ void C_Mutex::Lock()
   ASSERT(!iRc);         // Errors only happen with bad uses of the mutex
   USE(iRc);
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  EnterCriticalSection( &m_sMutex );
 #endif  
 }
 
@@ -571,8 +592,8 @@ void C_Mutex::UnLock()
   ASSERT(!iRc);         // Errors only happen with bad uses of the mutex
   USE(iRc);
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  LeaveCriticalSection( &m_sMutex );
 #endif  
 }
 
@@ -595,7 +616,7 @@ int C_Mutex::TryLock()
     return NO_ERR;
   }
 
-#elif defined WIN32
+#elif defined _WIN32
   ASSERT(false);
   return GEN_ERR;
 #endif
@@ -616,12 +637,18 @@ int C_Mutex::TryLock()
 C_Semaphore::C_Semaphore(unsigned int iInitialValue)
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
+# ifdef USE_SEM_T
   int iRc = sem_init(&sSemaphore, 0, iInitialValue);
   ASSERT(!iRc)          // Should never fail
   USE(iRc);
+# else
+  pthread_mutex_init(&m_sMutex, NULL);
+  pthread_cond_init(&m_sCondition, NULL);
+  sSemaphore = iInitialValue;
+# endif
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  sSemaphore = CreateSemaphore(NULL, iInitialValue, 32767, NULL);
 #endif  
 }
 
@@ -632,12 +659,17 @@ C_Semaphore::C_Semaphore(unsigned int iInitialValue)
 C_Semaphore::~C_Semaphore()
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
+# ifdef USE_SEM_T
   int iRc = sem_destroy(&sSemaphore);
   ASSERT(!iRc);         // No thread should be waiting on it at that time
   USE(iRc);
+# else
+  pthread_mutex_destroy(&m_sMutex);
+  pthread_cond_destroy(&m_sCondition);
+# endif
 
-#elif defined WIN32
-  ASSERT(false);
+#elif defined _WIN32
+  CloseHandle(sSemaphore);
 #endif
 }
 
@@ -648,11 +680,18 @@ C_Semaphore::~C_Semaphore()
 int C_Semaphore::Post()
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
+# ifdef USE_SEM_T
   return sem_post(&sSemaphore);
+# else
+  pthread_mutex_lock(&m_sMutex);
+  sSemaphore++;
+  pthread_cond_signal(&m_sCondition);
+  pthread_mutex_unlock(&m_sMutex);
+  return 0;
+# endif
 
-#elif defined WIN32
-  ASSERT(false);
-  return GEN_ERR;
+#elif defined _WIN32
+  return ReleaseSemaphore(sSemaphore, 1, NULL);
 #endif  
 }
 
@@ -663,11 +702,21 @@ int C_Semaphore::Post()
 int C_Semaphore::Wait()
 {
 #ifdef PTHREAD_COND_T_IN_PTHREAD_H
+# ifdef USE_SEM_T
   return sem_wait(&sSemaphore);
+# else
+  pthread_mutex_lock(&m_sMutex);
+  while (!sSemaphore)
+  {
+    pthread_cond_wait(&m_sCondition, &m_sMutex);
+  }
+  sSemaphore--;
+  pthread_mutex_unlock(&m_sMutex);
+  return 0;
+# endif
 
-#elif defined WIN32
-  ASSERT(false);
-  return GEN_ERR;
+#elif defined _WIN32
+  return WaitForSingleObject(sSemaphore, INFINITE);
 #endif
 }
 
@@ -690,8 +739,17 @@ C_Condition::C_Condition()
   iRc = pthread_cond_init(&m_sCondition, NULL);
   ASSERT(!iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  InitializeCriticalSection( &m_sMutex ); 
+  
+  /* initialise counter */
+  m_iWaitingThreads = 0;
+  /* Create an auto-reset event. */
+  m_sSignal = CreateEvent(NULL,  /* no security */
+                          FALSE, /* auto-reset event */
+                          FALSE, /* non-signaled initially */
+                          NULL); /* unnamed */
+  //ASSERT (!m_sSignal);
 #endif
 }
 
@@ -708,8 +766,8 @@ C_Condition::~C_Condition()
   iRc = pthread_cond_destroy(&m_sCondition);
   ASSERT(!iRc);         // No thread should wait on the condition
   
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  DeleteCriticalSection( &m_sMutex );
 #endif
 }
 
@@ -724,8 +782,8 @@ void C_Condition::Protect()
   ASSERT(!iRc);
   USE(iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  EnterCriticalSection( &m_sMutex );
 #endif
 }
 
@@ -740,8 +798,8 @@ void C_Condition::Release()
   ASSERT(!iRc);
   USE(iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  LeaveCriticalSection( &m_sMutex );
 #endif
 }
 
@@ -757,8 +815,14 @@ void C_Condition::Signal()
   iRc = pthread_mutex_unlock(&m_sMutex);
   ASSERT(!iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  int iWaitingThreads = m_iWaitingThreads;
+  while (m_iWaitingThreads
+         && m_iWaitingThreads == iWaitingThreads)
+  {
+    PulseEvent (m_sSignal);
+    Sleep (0); // deschedule the current thread 
+  }
 #endif
 }
 
@@ -774,8 +838,12 @@ void C_Condition::Broadcast()
   iRc = pthread_mutex_unlock(&m_sMutex);
   ASSERT(!iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  while (m_iWaitingThreads)
+  {
+    PulseEvent( m_sSignal );
+    Sleep( 0 ); // deschedule the current thread
+  }
 #endif
 }
 
@@ -790,8 +858,19 @@ void C_Condition::Wait()
   ASSERT(!iRc);
   USE(iRc);
 
-#else
-  ASSERT(false);
+#elif defined _WIN32
+  m_iWaitingThreads ++;
+
+  // Release the mutex
+  LeaveCriticalSection( &m_sMutex );
+
+  WaitForSingleObject( m_sSignal, INFINITE);
+
+  // maybe we should protect this with a mutex ?
+  m_iWaitingThreads --;
+
+  // Reacquire the mutex before returning.
+  EnterCriticalSection( &m_sMutex );
 #endif
 }
 
