@@ -2,7 +2,7 @@
 * netoutput.cpp: network output
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: netoutput.cpp,v 1.11.4.4 2003/03/08 01:38:37 nitrox Exp $
+* $Id: netoutput.cpp,v 1.11.4.5 2003/05/16 19:43:59 nitrox Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -29,6 +29,11 @@
 //------------------------------------------------------------------------------
 // Preamble
 //------------------------------------------------------------------------------
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
+
 #include "../../core/defs.h"
 #include "../../core/core.h"
 #include "../../mpeg/mpeg.h"
