@@ -2,7 +2,7 @@
 * ps2ts.cpp: MPEG1 and MPEG2 PS to MPEG2 TS converter
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: ps2ts.cpp,v 1.3 2002/04/27 01:17:44 asmax Exp $
+* $Id: ps2ts.cpp,v 1.4.4.1 2003/02/14 19:57:55 nitrox Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -51,7 +51,9 @@ template <class Reader, class TsProvider>
  C_Ps2Ts<Reader, TsProvider>::C_Ps2Ts(Reader* pReader, TsProvider* pTsProvider,
                                       unsigned int iMaxBufferedTs,
                                       unsigned int iMpegVersion) :
-                             m_cPgrmDescriptor(iMpegVersion), m_cPendingTS(NO)
+                             m_cPgrmDescriptor(iMpegVersion),
+                             m_cPat(0, 0, 0), m_cPmt(0, 0, 0, 0, 0x50),
+                             m_cPendingTS(NO)
 {
   ASSERT(pReader);
   ASSERT(pTsProvider);
@@ -62,13 +64,15 @@ template <class Reader, class TsProvider>
   m_pTsProvider = pTsProvider;
   m_iMaxBufferedTs = iMaxBufferedTs;
 
+  m_iPrevSCR = 0;
   m_iDataType = UNKNOWN_DATA;
   m_iStatus = NO_ERR;
+  m_pDelayedPacket = NULL;
   m_bDiscontinuity = false;
 
   ZERO(m_pCurrentData);
 
-  m_bSendPSI = false;
+  m_bSendPSI = m_bGenPat = m_bGenPmt = m_bResetPSI = true;
   m_iTSCounter = 0;
 
   if(iMpegVersion == 1)
@@ -155,56 +159,106 @@ template <class Reader, class TsProvider>
 {
   if(m_cPendingTS.Size() == 0)
   {
-    if(m_bSendPSI)
+    if(!m_bSendPSI)
     {
-      m_bSendPSI = false;
-      m_iTSCounter += 2;
-
-//      printf("!!! Sending PSI !!!\n");
-      ASSERT(m_iMaxBufferedTs >= 2);
-
-      // Push the PAT
-      C_TsPacket* pPacket = m_pTsProvider->GetPacket();
-      ASSERT(pPacket);
-      m_cPgrmDescriptor.WritePAT(pPacket);
-      m_cPendingTS.PushEnd(pPacket);
-
-      // Now push the PMT
-      ZERO(pPacket);
-      pPacket = m_pTsProvider->GetPacket();
-      ASSERT(pPacket);
-      m_cPgrmDescriptor.WritePMT(pPacket);
-      m_cPendingTS.PushEnd(pPacket);
-    }
-    else
-    {
-      int iRc = FetchPackets();
-      if(iRc)
+      if(m_pDelayedPacket)
       {
-        do
-        {
-          if(iRc == UNKNOWN_DATA)
-          {
-            iRc = Synch();
-            if(!iRc)
-              iRc = FetchPackets();
-          }
-          else if(iRc == SKIPPED_DATA)
-            iRc = FetchPackets();
-        }
-        while(iRc == SKIPPED_DATA || iRc == UNKNOWN_DATA);
+        m_cPendingTS.PushEnd(m_pDelayedPacket);
+        m_pDelayedPacket = NULL;
       }
       else
       {
-        // Check if we will have to insert PSI on the next iteration: we
-        // add the PSI in the stream every half second
-        // (For 1.5 Mbps streams, about 1000 TS packets are sent every second)
-        m_bSendPSI = (m_iTSCounter % 500 <= m_iMaxBufferedTs);
-//        printf("mitscounter: %d\n", m_iTSCounter);
+        int iRc = FetchPackets();
+        if(iRc && !m_bSendPSI)
+        {
+          do
+          {
+            if(iRc == UNKNOWN_DATA)
+            {
+              iRc = Synch();
+              if(!iRc)
+                iRc = FetchPackets();
+            }
+            else if(iRc == SKIPPED_DATA)
+              iRc = FetchPackets();
+          }
+          while((iRc == SKIPPED_DATA || iRc == UNKNOWN_DATA) && !m_bSendPSI);
+        }
+        else if(!m_bSendPSI)
+        {
+          // Check if we will have to insert PSI on the next iteration: we
+          // add the PSI in the stream every half second
+          // (For 1.5 Mbps streams, about 1000 TS packets are sent every second)
+          m_bSendPSI = m_bResetPSI = (m_iTSCounter % 500 <= m_iMaxBufferedTs);
+//          printf("mitscounter: %d\n", m_iTSCounter);
+        }
+      }
+    }
+
+    if(m_bSendPSI)
+    {
+      if(m_bGenPat)
+      {
+        *m_cPat.GetLowLevelPat() = m_cPgrmDescriptor.m_sPat;
+        m_cPat.Generate();
+        m_cPat.GetLowLevelPat()->p_first_program = NULL;
+        m_bGenPat = false;
+      }
+
+      if(m_bGenPmt)
+      {
+        m_cPgrmDescriptor.m_sPmt.i_version =
+                        (m_cPgrmDescriptor.m_sPmt.i_version + 1) & 0x1f;
+        *m_cPmt.GetLowLevelPmt() = m_cPgrmDescriptor.m_sPmt;
+        m_cPmt.Generate();
+        m_cPmt.GetLowLevelPmt()->p_first_es = NULL;
+        m_cPmt.GetLowLevelPmt()->p_first_descriptor = NULL;
+        m_bGenPmt = false;
+      }
+
+      if(m_bResetPSI)
+      {
+        m_cPat.TsReset();
+        m_cPmt.TsReset();
+        m_bResetPSI = false;
+      }
+
+      for(unsigned int i = m_cPendingTS.Size(); i < m_iMaxBufferedTs; i++)
+      {
+        C_TsPacket* pPacket = m_pTsProvider->GetPacket();
+        ASSERT(pPacket);
+
+        if(m_cPat.TsHasNext())
+        {
+          m_cPat.TsWrite(pPacket);
+          m_cPendingTS.PushEnd(pPacket);
+          m_iTSCounter++;
+        }
+        else if(m_cPmt.TsHasNext())
+        {
+          m_cPmt.TsWrite(pPacket);
+          m_cPendingTS.PushEnd(pPacket);
+          m_iTSCounter++;
+        }
+        else
+        {
+          m_bSendPSI = false;
+          m_pTsProvider->ReleasePacket(pPacket);
+          if(m_pDelayedPacket)
+          {
+            m_cPendingTS.PushEnd(m_pDelayedPacket);
+            m_pDelayedPacket = NULL;
+          }
+        }
+        if(!m_cPat.TsHasNext() && !m_cPmt.TsHasNext())
+          m_bSendPSI = false;
       }
     }
   }
 
+  ASSERT(m_cPendingTS.Size() > 0);
+  ASSERT(m_cPendingTS.Size() <= m_iMaxBufferedTs);
+
   // Return the first packet
   if(m_cPendingTS.Size() > 0)
     return m_cPendingTS.Remove(0);
@@ -265,7 +319,7 @@ template <class Reader, class TsProvider>
 //        printf("Program map pes encountered\n");
         m_iStatus = ParsePgrmMap(pPacket, &iPosInTs);
         // A foutre dans parse pgrm_map_pes quand le pes est completement fini
-        m_bSendPSI = true;
+        m_bSendPSI = m_bResetPSI = true;
         break;
       }
       case PES_H_PRIVATE_2:
@@ -591,14 +645,26 @@ template <class Reader, class TsProvider>
     // We are no more at the beginning of the PES yet
     m_bPESStart = false;
 
-    // Put the TS packet in the list of pending TS
-    m_cPendingTS.PushEnd(pPacket);
-    ZERO(pPacket);
-
     // Increase the number of TS packets that have been build
     m_pCurrentData->IncreaseCounter();
     m_iTSCounter++;
 
+    if(m_cPgrmDescriptor.m_bNewPmt)
+    {
+      // Delay this packet to send PSI first
+      ASSERT(!m_pDelayedPacket);
+      m_pDelayedPacket = pPacket;
+      m_bSendPSI = m_bGenPmt = m_bResetPSI = true;
+      m_cPgrmDescriptor.m_bNewPmt = false;
+      i = m_iMaxBufferedTs - 1;
+    }
+    else
+    {
+      // Put the TS packet in the list of pending TS
+      m_cPendingTS.PushEnd(pPacket);
+    }
+    ZERO(pPacket);
+
     // Prepare the next iteration if we must loop
     if(m_iDataLen > 0)
     {
